/*

 wmirq.c
 Interrupts Monitor
 by Sbastien LIENARD (slienard@worldnet.fr), GCU (http://gcu.home.ml.org)
 havyly pached by Lennart Poettering (poettering@gmx.net)
 
 heavily based on wminet by Dave Clark (clarkd@skyia.com) and Antoine
 Nulle (warp@xs4all.nl) and  wmifs by Martijn Pieterse (pieterse@xs4all.nl).

 see http://windowmaker.mezaway.org for more awesome wm dock apps :)

*/


#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"

#include "wmirq-master.xpm"
#include "wmirq-mask.xbm"

#define APP_VERSION "0.1"

char *appname;

typedef struct 
 {
  int bit;
  int irqs;
  char *label;
 } INTERRUPT;

INTERRUPT interrupts[16];

void getData(void)
 {
  int i, lastirqs;
  FILE *fp;
  char tokens[]= " :+";
  char buf[128], s[256], *l, *t;

  fp = fopen("/proc/interrupts","r");
  if (fp)
   {
    fgets(buf, 128, fp); 
   
    for (i = 0; i < 16; i++)
     {
      if (interrupts[i].label)
       free(interrupts[i].label);
      interrupts[i].bit = 0; 
     }  
   
    while ((fgets(buf, 128, fp)))
     {
      t = strtok(buf, tokens);
      if (strcmp(t, "NMI") != 0)
       {
        i = atoi(t);
        lastirqs = interrupts[i].irqs;
        interrupts[i].irqs = atoi (strtok(NULL, tokens));
        strtok(NULL, tokens);
		s[0] = 0;
        do
	     {
 	      l = strtok(NULL, tokens);
 	      if (l) strcat(s, l);
	     } 
	    while (l); 
	 
        if (strchr(s, '\n')) *strchr(s, '\n') = 0;
        interrupts[i].label = strdup (s);
        interrupts[i].bit = (interrupts[i].irqs != lastirqs) ? 1 : 0;
       }    	
     }
    fclose (fp);
   }
  else 
   {
    fprintf(stderr, "Can't open /proc/interrupts.\n");
    exit(2);
   } 
 }

void blitString(char *name, int x, int y)
 {
  int i, c, k, m;

  m = strlen(name);
  for (i=0; i < 9; i++)
   {
    if (i < m) c = toupper(name[i]); else c = ' '; 
      
    k = x+i*6;
    if (c >= 'A' && c <= 'Z')
     copyXPMArea((c-'A') * 6, 74, 6, 8, k, y);
    else if (c >= '0' && c <= '9')
	 copyXPMArea((c-'0') * 6, 64, 6, 8, k, y);
    else if (c == ' ')
	 copyXPMArea(12 * 6, 64, 6, 8, k, y);
   }
 }

void blitNum(int num, int x, int y, int r)
 {
  char buf[1024], buf2[1024];
  sprintf(buf, "%%%ii", r);
  sprintf(buf2, buf, num);
  blitString(buf2, x, y);
 }
    
void displayLabel (char *label)
 {
  copyXPMArea(4, 84, 39, 10, 4, 21);
  copyXPMArea(4, 84, 55, 10, 4, 35);
  blitString (label ? label : "unused", 4, 35);  
 }

void mainRoutine(int argc, char **argv)
 {
  XEvent Event;
  int i, but_stat = -1;
  int index = 0, quit = 0;

  memset(interrupts, 0, sizeof(interrupts));

  openXwindow(argc, argv, wmirq_master_xpm, wmirq_mask_bits, wmirq_mask_width, wmirq_mask_height);

  index = 0;
  for (i = 5; i < 64-5; i+=7)
   {
    copyXPMArea(81, 66, 4, 4,  i,  5);
    AddMouseRegion(index++, i, 5, i+5, 5+5);
   } 

  for (i = 5; i < 64-5; i+=7)
   {
    copyXPMArea(81, 66, 4, 4,  i, 12);
    AddMouseRegion(index++, i, 12, i+5, 12+5);
   }
  
  AddMouseRegion(16,  5, 21, 59, 45);
  RedrawWindow();
  blitNum (0, 46, 21, 2);
  
  index = 0;

  while (!quit)
   {
    getData();
    displayLabel (interrupts[index].label);
    blitNum (interrupts[index].irqs, 4, 49, 9); 

    for (i=0; i<8; i++)
     copyXPMArea(interrupts[i].bit ? 87 : 93, 66, 4, 4, (5+i*7), 5);
    for (i=8; i<16; i++)
     copyXPMArea(interrupts[i].bit ? 87 : 93, 66, 4, 4, (7*i-51), 12);

    RedrawWindow();

    while (XPending(display))
     {
      XNextEvent(display, &Event);
      switch (Event.type)
       {
 		case Expose:
  		 RedrawWindow();
		 break;
		case DestroyNotify:
		 XCloseDisplay(display);
         quit = 1;
		 break;
		case ButtonPress:
		 but_stat = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
		 break;
		case ButtonRelease:
		 i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
         if ((but_stat == i) && (i >= 0))
          {
           if (but_stat == 16) 
            {
             index++;
             if (index > 15) index = 0;
            }             
           else 
            index = but_stat;
           blitNum (index, 46, 21, 2);
          } 

         but_stat = -1;
         break;
       }
     }
	usleep(50000L);
   }
 }


void printVersion(void)
 {
  printf("%s v%s\n", appname, APP_VERSION);
  exit(0);
 }

void printLicence(void)
 {
  printf("%s v%s -- Dockable IRQ monitor for Window Maker.\n", appname, APP_VERSION);
  printf("Based on wmint by Sbastien LIENARD. Patched by Lennart Poettering <poettering@gmx.net>.\n");
  printf("This software comes with NO WARRANTY: see the file COPYING for details.\n");
 }

void usage(void)
 {
  printLicence();
  printf("Usage: %s [-d <display>]\n", appname);
  printf("       %s -v\n", appname);
  printf("       %s -h\n\n", appname);
  printf("          -d Selects X11-display.\n");
  printf("          -h This help screen\n");
  printf("          -v Print the version number\n");
  exit(1);
 }

int main(int argc, char *argv[]) 
 {
  int i;
  appname = strdup(argv[0]);

  for (i=1; i < argc; i++) 
   {
	char *arg = argv[i];

	if (*arg=='-') 
     {
   	  switch (arg[1]) 
       {
	    case 'd' :
		 if (strcmp(arg+1, "display")) usage();
   	     break;
        case 'v' :
		 printVersion();
		 break;
		default:
		 usage();
         break;
	   }
	 }
   }  

  printLicence();
  mainRoutine(argc, argv);
    
  if (appname)  
   free(appname);
   
  return 0; 
 }
